/* BdfServer_Commands - Copyright (c) 2011-2021 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.commands.corpus;

import fr.exemole.bdfserver.api.BdfServer;
import fr.exemole.bdfserver.api.BdfServerEditor;
import fr.exemole.bdfserver.api.instruction.BdfInstructionException;
import fr.exemole.bdfserver.api.ui.UiComponents;
import fr.exemole.bdfserver.tools.instruction.AbstractBdfCommand;
import fr.exemole.bdfserver.tools.instruction.BdfInstructionUtils;
import fr.exemole.bdfserver.tools.ui.components.IncludeUiBuilder;
import java.text.ParseException;
import net.fichotheque.Subset;
import net.fichotheque.SubsetKey;
import net.fichotheque.corpus.Corpus;
import net.fichotheque.include.ExtendedIncludeKey;
import net.fichotheque.include.IncludeKey;
import net.mapeadores.util.request.RequestMap;
import net.mapeadores.util.text.StringUtils;


/**
 *
 * @author Vincent Calame
 */
public class SubsetIncludeCreationCommand extends AbstractBdfCommand {

    public final static String COMMANDNAME = "SubsetIncludeCreation";
    public final static String COMMANDKEY = "_ CRP-12";
    public final static String SUBSET_PARAMNAME = "subset";
    public final static String MASTERINCLUDE_PARAMNAME = "masterinclude";
    public final static String POIDSFILTER_PARAMNAME = "poidsfilter";
    public final static String MODE_PARAMNAME = "mode";
    public final static String NO_POIDSFILTER_PARAMVALUE = "0";
    public final static String YES_POIDSFILTER_PARAMVALUE = "1";
    public final static String POIDSFILTERVALUE_PARAMNAME = "poidsfiltervalue";
    private Corpus corpus;
    private UiComponents uiComponents;
    private Subset subset;
    private ExtendedIncludeKey includeKey;

    public SubsetIncludeCreationCommand(BdfServer bdfServer, RequestMap requestMap) {
        super(bdfServer, requestMap);
    }

    @Override
    public boolean needSynchronisation() {
        return true;
    }

    @Override
    protected void doCommand() {
        putResultObject(CORPUS_OBJ, corpus);
        BdfServerEditor bdfServerEditor = bdfServer.getBdfServerEditor();
        boolean done = bdfServerEditor.putComponentUi(uiComponents, IncludeUiBuilder.toDefaultSubsetIncludeUi(includeKey));
        if (done) {
            bdfServerEditor.saveChanges();
            setDone("_ done.corpus.subsetincludecreation", includeKey);
        }
    }

    @Override
    protected boolean checkParameters() throws BdfInstructionException {
        corpus = getMandatoryCorpus();
        checkSubsetAdmin(corpus);
        String subsetString = getMandatory(SUBSET_PARAMNAME);
        SubsetKey subsetKey = null;
        try {
            subsetKey = SubsetKey.parse(subsetString);
        } catch (java.text.ParseException pe) {
            throw new BdfInstructionException(BdfInstructionUtils.wrongParameterValue(SUBSET_PARAMNAME, subsetString));
        }
        subset = bdfServer.getFichotheque().getSubset(subsetKey);
        if (subset == null) {
            throw new BdfInstructionException(BdfInstructionUtils.unknownParameterValue(SUBSET_PARAMNAME, subsetString));
        }
        String poidsFilterString = getMandatory(POIDSFILTER_PARAMNAME);
        int poidsFilter = -1;
        if (poidsFilterString.equals(YES_POIDSFILTER_PARAMVALUE)) {
            String poidsFilterValue = getMandatory(POIDSFILTERVALUE_PARAMNAME);
            boolean error = false;
            try {
                poidsFilter = Integer.parseInt(poidsFilterValue);
                if (poidsFilter < 1) {
                    error = true;
                }
            } catch (NumberFormatException nfe) {
                error = true;
            }
            if (error) {
                setError("_ error.wrong.poids", poidsFilterValue);
                return false;
            }
        } else if (!poidsFilterString.equals(NO_POIDSFILTER_PARAMVALUE)) {
            throw new BdfInstructionException(BdfInstructionUtils.unknownParameterValue(POIDSFILTER_PARAMNAME, poidsFilterString));
        }
        String mode = requestMap.getParameter(MODE_PARAMNAME);
        if (mode != null) {
            mode = mode.trim();
            if (mode.length() > 0) {
                try {
                    StringUtils.checkTechnicalName(mode, false);
                } catch (ParseException pe) {
                    setError("_ error.wrong.mode", mode);
                    return false;
                }
            }
        } else {
            mode = "";
        }
        boolean master = false;
        Subset masterSubset = corpus.getMasterSubset();
        if (masterSubset != null) {
            master = requestMap.isTrue(MASTERINCLUDE_PARAMNAME);
        }
        try {
            includeKey = ExtendedIncludeKey.newInstance(IncludeKey.newInstance(subsetKey, mode, poidsFilter), master);
        } catch (IllegalArgumentException iae) {
            throw new BdfInstructionException(BdfInstructionUtils.unknownParameterValue(SUBSET_PARAMNAME, subsetString));
        }
        uiComponents = bdfServer.getUiManager().getMainUiComponents(corpus);
        if (uiComponents.contains(includeKey.toString())) {
            setError("_ error.existing.include", includeKey.toString());
            return false;
        }
        return true;
    }

}
