/* FichothequeLib_API - Copyright (c) 2007-2021 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.fichotheque.utils.selection;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Set;
import java.util.function.Predicate;
import net.fichotheque.Fichotheque;
import net.fichotheque.Subset;
import net.fichotheque.SubsetKey;
import net.fichotheque.corpus.FicheMeta;
import net.fichotheque.croisement.Croisement;
import net.fichotheque.croisement.Croisements;
import net.fichotheque.selection.CroisementCondition;
import net.fichotheque.selection.MotcleCondition;
import net.fichotheque.selection.MotcleQuery;
import net.fichotheque.selection.MotcleSelector;
import net.fichotheque.selection.SelectionContext;
import net.fichotheque.thesaurus.Motcle;
import net.fichotheque.thesaurus.Thesaurus;
import net.fichotheque.utils.SelectionUtils;
import net.mapeadores.util.conditions.Condition;
import net.mapeadores.util.conditions.ConditionsConstants;
import net.mapeadores.util.conditions.ConditionsUtils;
import net.mapeadores.util.conditions.TextTestEngine;
import net.mapeadores.util.exceptions.SwitchException;
import net.mapeadores.util.text.Label;


/**
 *
 * @author Vincent Calame
 */
public final class MotcleConditionPredicateFactory {

    private final static Test ALWAYS_TEST = new ConstantTest(true);

    private MotcleConditionPredicateFactory() {

    }

    public static Predicate<FicheMeta> newInstance(SelectionContext selectionContext, MotcleCondition motcleCondition) {
        Test[] testArray;
        List<Test> testList = new ArrayList<Test>();
        for (MotcleCondition.Entry entry : motcleCondition.getEntryList()) {
            Test test = toTest(selectionContext, entry.getMotcleQuery(), entry.getCroisementCondition());
            if (test != null) {
                testList.add(test);
            }
        }
        int size = testList.size();
        if (size == 0) {
            testArray = new Test[1];
            testArray[0] = ALWAYS_TEST;
        } else {
            testArray = testList.toArray(new Test[size]);
        }
        return new ResultPredicate(motcleCondition.getSelectMode(), testArray);
    }


    private static class ResultPredicate implements Predicate<FicheMeta> {

        private final short optionsMode;
        private final Test[] testArray;

        private ResultPredicate(short mode, Test[] testArray) {
            this.optionsMode = mode;
            this.testArray = testArray;
        }

        @Override
        public boolean test(FicheMeta ficheMeta) {
            if (optionsMode == ConditionsConstants.MODE_OR) {
                for (Test test : testArray) {
                    if (test.isSelected(ficheMeta)) {
                        return true;
                    }
                }
                return false;
            } else {
                for (Test test : testArray) {
                    if (!test.isSelected(ficheMeta)) {
                        return false;
                    }
                }
                return true;
            }
        }

    }

    private static Test toTest(SelectionContext selectionContext, MotcleQuery motcleQuery, CroisementCondition croisementCondition) {
        boolean invert = false;
        if (SelectionUtils.isEmpty(motcleQuery, croisementCondition)) {
            return null;
        } else if (SelectionUtils.isOnlyList(motcleQuery, croisementCondition)) {
            return new ThesaurusListTest(invert, selectionContext, motcleQuery);
        } else if (motcleQuery.getContentCondition() == null) {
            return new MotcleSelectorTest(invert, selectionContext, motcleQuery, croisementCondition);
        } else {
            Condition fieldCondition = motcleQuery.getContentCondition();
            short fieldScope = motcleQuery.getContentConditionScope();
            boolean withSelector = false;
            if (hasSelectorConditions(motcleQuery, croisementCondition)) {
                withSelector = true;
                motcleQuery = SelectionUtils.deriveWithoutContentCondition(motcleQuery);
            }
            int conditionType = ConditionsUtils.getConditionType(fieldCondition);
            switch (conditionType) {
                case ConditionsConstants.IMPOSSIBLE_CONDITION:
                    return new ConstantTest(!invert);
                case ConditionsConstants.NEUTRAL_CONDITION:
                case ConditionsConstants.UNKNOWN_CONDITION:
                    if (withSelector) {
                        return new MotcleSelectorTest(invert, selectionContext, motcleQuery, croisementCondition);
                    } else {
                        return new ConstantTest(invert);
                    }
                case ConditionsConstants.NOTEMPTY_CONDITION:
                    if (withSelector) {
                        return new MotcleSelectorTest(invert, selectionContext, motcleQuery, croisementCondition);
                    } else {
                        return new ThesaurusListTest(invert, selectionContext, motcleQuery);
                    }
                case ConditionsConstants.EMPTY_CONDITION:
                    if (withSelector) {
                        return new MotcleSelectorTest(!invert, selectionContext, motcleQuery, croisementCondition);
                    } else {
                        return new ThesaurusListTest(!invert, selectionContext, motcleQuery);
                    }
                case ConditionsConstants.PARTIAL_CONDITION:
                case ConditionsConstants.PARTIALOREMPTY_CONDITION:
                    return new FieldTest(invert, selectionContext, motcleQuery, croisementCondition, fieldCondition, fieldScope, conditionType, withSelector);
                default:
                    throw new SwitchException("state = " + conditionType);
            }
        }
    }

    private static Set<SubsetKey> toSubsetKeySet(Thesaurus[] thesaurusArray) {
        Set<SubsetKey> subsetKeySet = new HashSet<SubsetKey>();
        for (Thesaurus thesaurus : thesaurusArray) {
            subsetKeySet.add(thesaurus.getSubsetKey());
        }
        return subsetKeySet;
    }

    private static boolean hasSelectorConditions(MotcleQuery motcleQuery, CroisementCondition croisementCondition) {
        if (motcleQuery.getIdRanges() != null) {
            return true;
        }
        if (motcleQuery.getLevelRanges() != null) {
            return true;
        }
        if (motcleQuery.getFicheCondition() != null) {
            return true;
        }
        if (croisementCondition != null) {
            return true;
        }
        return false;
    }


    private static abstract class Test {

        private final boolean invert;

        protected Test(boolean invert) {
            this.invert = invert;
        }

        public boolean isSelected(FicheMeta ficheMeta) {
            boolean test = test(ficheMeta);
            if (invert) {
                return !test;
            } else {
                return test;
            }
        }

        protected abstract boolean test(FicheMeta ficheMeta);

    }


    private static class ConstantTest extends Test {

        public ConstantTest(boolean constant) {
            super(constant);
        }

        @Override
        public boolean test(FicheMeta ficheMeta) {
            return true;
        }

    }


    private static class ThesaurusListTest extends Test {

        private final Fichotheque fichotheque;
        private final Thesaurus[] thesaurusArray;
        private final boolean withSatelliteTest;
        private final Set<SubsetKey> subsetKeySet;

        public ThesaurusListTest(boolean invert, SelectionContext selectionContext, MotcleQuery motcleQuery) {
            super(invert);
            this.fichotheque = selectionContext.getFichotheque();
            this.thesaurusArray = SelectionUtils.toThesaurusArray(fichotheque, motcleQuery, selectionContext.getSubsetAccessPredicate());
            this.withSatelliteTest = motcleQuery.includeSatellites();
            if (withSatelliteTest) {
                this.subsetKeySet = toSubsetKeySet(thesaurusArray);
            } else {
                this.subsetKeySet = null;
            }
        }

        @Override
        public boolean test(FicheMeta ficheMeta) {
            if (withSatelliteTest) {
                Subset masterSubset = ficheMeta.getCorpus().getMasterSubset();
                if (masterSubset != null) {
                    if (subsetKeySet.contains(masterSubset.getSubsetKey())) {
                        return true;
                    }
                }
            }
            for (Thesaurus thesaurus : thesaurusArray) {
                Croisements croisements = fichotheque.getCroisements(ficheMeta, thesaurus);
                if (!croisements.isEmpty()) {
                    return true;
                }
            }
            return false;
        }

    }


    private static class MotcleSelectorTest extends Test {

        private final Fichotheque fichotheque;
        private final Thesaurus[] thesaurusArray;
        private final boolean withSatelliteTest;
        private final Set<SubsetKey> subsetKeySet;
        private final MotcleSelector motcleSelector;
        private final boolean withCroisementCondition;

        public MotcleSelectorTest(boolean invert, SelectionContext selectionContext, MotcleQuery motcleQuery, CroisementCondition croisementCondition) {
            super(invert);
            this.fichotheque = selectionContext.getFichotheque();
            this.thesaurusArray = SelectionUtils.toThesaurusArray(fichotheque, motcleQuery, selectionContext.getSubsetAccessPredicate());
            this.withSatelliteTest = motcleQuery.includeSatellites();
            if (withSatelliteTest) {
                this.subsetKeySet = toSubsetKeySet(thesaurusArray);
            } else {
                this.subsetKeySet = null;
            }
            this.motcleSelector = MotcleSelectorBuilder.init(selectionContext).add(motcleQuery, croisementCondition).toMotcleSelector();
            this.withCroisementCondition = (croisementCondition != null);
        }

        @Override
        public boolean test(FicheMeta ficheMeta) {
            if (withSatelliteTest) {
                Subset masterSubset = ficheMeta.getCorpus().getMasterSubset();
                if (masterSubset != null) {
                    if (subsetKeySet.contains(masterSubset.getSubsetKey())) {
                        if (motcleSelector.test((Motcle) masterSubset.getSubsetItemById(ficheMeta.getId()))) {
                            return true;
                        }
                    }
                }
            }
            for (Thesaurus thesaurus : thesaurusArray) {
                Croisements croisements = fichotheque.getCroisements(ficheMeta, thesaurus);
                for (Croisements.Entry entry : croisements.getEntryList()) {
                    Motcle motcle = (Motcle) entry.getSubsetItem();
                    if (!withCroisementCondition) {
                        if (motcleSelector.test(motcle)) {
                            return true;
                        }
                    } else {
                        if (motcleSelector.isSelected(motcle, entry.getCroisement()) != null) {
                            return true;
                        }
                    }
                }
            }
            return false;
        }

    }


    private static class FieldTest extends Test {

        private final Fichotheque fichotheque;
        private final Thesaurus[] thesaurusArray;
        private final boolean withSatelliteTest;
        private final Set<SubsetKey> subsetKeySet;
        private final TextTestEngine textTestEngine;
        private final boolean allowsEmpty;
        private final boolean withSelector;
        private final MotcleSelector motcleSelector;
        private final boolean withCroisementCondition;
        private final short fieldScope;


        public FieldTest(boolean invert, SelectionContext selectionContext, MotcleQuery motcleQuery, CroisementCondition croisementCondition, Condition fieldCondition, short fieldScope, int conditionType, boolean withSelector) {
            super(invert);
            this.fichotheque = selectionContext.getFichotheque();
            this.thesaurusArray = SelectionUtils.toThesaurusArray(fichotheque, motcleQuery, selectionContext.getSubsetAccessPredicate());
            this.withSatelliteTest = motcleQuery.includeSatellites();
            if (withSatelliteTest) {
                this.subsetKeySet = toSubsetKeySet(thesaurusArray);
            } else {
                this.subsetKeySet = null;
            }
            this.textTestEngine = TextTestEngine.newInstance(fieldCondition, selectionContext.getWorkingLang());
            this.allowsEmpty = ((!withSelector) && (conditionType == ConditionsConstants.PARTIALOREMPTY_CONDITION));
            this.withSelector = withSelector;
            if (withSelector) {
                this.motcleSelector = MotcleSelectorBuilder.init(selectionContext).add(motcleQuery, croisementCondition).toMotcleSelector();
            } else {
                this.motcleSelector = null;
            }
            this.withCroisementCondition = (croisementCondition != null);
            this.fieldScope = fieldScope;
        }


        @Override
        public boolean test(FicheMeta ficheMeta) {
            boolean empty = true;
            textTestEngine.start();
            if (withSatelliteTest) {
                Subset masterSubset = ficheMeta.getCorpus().getMasterSubset();
                if (masterSubset != null) {
                    if (subsetKeySet.contains(masterSubset.getSubsetKey())) {
                        Motcle motcle = (Motcle) masterSubset.getSubsetItemById(ficheMeta.getId());
                        if (checkSelector(motcle, null)) {
                            empty = false;
                            checkTextTest(motcle);
                            if (textTestEngine.canStop()) {
                                return textTestEngine.getResult();
                            }
                        }
                    }
                }
            }
            for (Thesaurus thesaurus : thesaurusArray) {
                Croisements croisements = fichotheque.getCroisements(ficheMeta, thesaurus);
                for (Croisements.Entry entry : croisements.getEntryList()) {
                    Motcle motcle = (Motcle) entry.getSubsetItem();
                    if (checkSelector(motcle, entry.getCroisement())) {
                        empty = false;
                        checkTextTest(motcle);
                        if (textTestEngine.canStop()) {
                            return textTestEngine.getResult();
                        }
                    }
                }
            }
            if (empty) {
                return allowsEmpty;
            } else {
                return textTestEngine.getResult();
            }
        }

        private boolean checkSelector(Motcle motcle, Croisement croisement) {
            if (!withSelector) {
                return true;
            }
            if ((croisement == null) || (!withCroisementCondition)) {
                return motcleSelector.test(motcle);
            } else {
                return (motcleSelector.isSelected(motcle, croisement) != null);
            }
        }

        private void checkTextTest(Motcle motcle) {
            if (fieldScope != MotcleQuery.WITHOUT_IDALPHA_SCOPE) {
                String idalpha = motcle.getIdalpha();
                if (idalpha != null) {
                    textTestEngine.addString(idalpha);
                    if (fieldScope == MotcleQuery.ONLY_IDALPHA_SCOPE) {
                        return;
                    }
                }
            }
            Label label = motcle.getLabels().getLabel(textTestEngine.getLang());
            if (label != null) {
                textTestEngine.addString(label.getLabelString());
            }
        }

    }

}
