/* FichothequeLib_Tools - Copyright (c) 2010-2021 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.fichotheque.tools.extraction.builders;

import java.util.AbstractList;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.RandomAccess;
import net.fichotheque.corpus.Corpus;
import net.fichotheque.corpus.FicheMeta;
import net.fichotheque.corpus.Fiches;
import net.fichotheque.extraction.ExtractionConstants;
import net.fichotheque.extraction.ExtractionContext;
import net.fichotheque.extraction.ExtractionSource;
import net.fichotheque.extraction.def.DocumentFilter;
import net.fichotheque.extraction.def.ExtractionDef;
import net.fichotheque.extraction.def.ExtraitAddendaDef;
import net.fichotheque.extraction.def.ExtraitAlbumDef;
import net.fichotheque.extraction.def.ExtraitCorpusDef;
import net.fichotheque.extraction.def.ExtraitThesaurusDef;
import net.fichotheque.extraction.def.FicheFilter;
import net.fichotheque.extraction.def.FicheRattachementDef;
import net.fichotheque.extraction.def.GroupClause;
import net.fichotheque.extraction.def.IllustrationFilter;
import net.fichotheque.extraction.def.IntituleClause;
import net.fichotheque.extraction.def.MotcleFilter;
import net.fichotheque.extraction.def.PackClause;
import net.fichotheque.extraction.run.CorpusExtractor;
import net.fichotheque.extraction.run.ExtraitCorpusResult;
import net.fichotheque.extraction.run.ExtraitThesaurusResult;
import net.fichotheque.extraction.run.ThesaurusExtractor;
import net.fichotheque.selection.DocumentCondition;
import net.fichotheque.selection.FicheCondition;
import net.fichotheque.selection.FicheSelector;
import net.fichotheque.selection.IllustrationCondition;
import net.fichotheque.selection.MotcleCondition;
import net.fichotheque.selection.MotcleSelector;
import net.fichotheque.selection.SelectionContext;
import net.fichotheque.thesaurus.Motcle;
import net.fichotheque.thesaurus.Thesaurus;
import net.fichotheque.utils.CorpusUtils;
import net.fichotheque.utils.SelectionUtils;
import net.fichotheque.utils.selection.FicheSelectorBuilder;
import net.fichotheque.utils.selection.MotcleSelectorBuilder;
import net.fichotheque.utils.selection.SelectionContextBuilder;
import net.mapeadores.util.localisation.Lang;
import net.mapeadores.util.localisation.LocalisationUtils;


/**
 *
 * @author Vincent Calame
 */
public final class BuilderUtils {

    public final static List<ExtraitCorpusDef> EMPTY_EXTRAITCORPUSDEFLIST = Collections.emptyList();
    public final static List<ExtraitThesaurusDef> EMPTY_EXTRAITTHESAURUSDEFLIST = Collections.emptyList();
    public final static List<ExtraitCorpusResult> EMPTY_EXTRAITCORPUSRESULTLIST = Collections.emptyList();
    public final static List<ExtraitThesaurusResult> EMPTY_EXTRAITTHESAURUSRESULTLIST = Collections.emptyList();
    public final static List<FicheRattachementDef> EMPTY_FICHERATTACHEMENTDEFLIST = Collections.emptyList();

    private BuilderUtils() {
    }

    public static String testTagName(String tagName, String defaultTagName) {
        if (tagName == null) {
            return defaultTagName;
        }
        if (tagName.length() == 0) {
            return defaultTagName;
        }
        if (tagName.equals("NULL")) {
            return null;
        }
        if (testCharacters(tagName)) {
            return tagName;
        }
        return defaultTagName;
    }

    public static ExtractionSource getExtractionSource(Object dynamicObject) {
        return new SimpleExtractionSource(dynamicObject);
    }

    public static ExtractionSource getExtractionSource(Object dynamicObject, ExtractionContext extractionContext, ExtractionDef extractionDef) {
        List<ExtraitCorpusDef> extraitCorpusDefList = extractionDef.getStaticExtraitCorpusDefList();
        List<ExtraitThesaurusDef> extraitThesaurusDefList = extractionDef.getStaticExtraitThesaurusDefList();
        if ((extraitCorpusDefList.isEmpty()) && (extraitThesaurusDefList.isEmpty())) {
            return new SimpleExtractionSource(dynamicObject);
        }
        List<ExtraitCorpusResult> extraitCorpusResultList = runExtraitCorpusDef(extraitCorpusDefList, extractionContext);
        List<ExtraitThesaurusResult> extraitThesaurusResultList = runExtraitThesaurusDef(extraitThesaurusDefList, extractionContext);
        return new CompleteExtractionSource(dynamicObject, extraitCorpusResultList, extraitThesaurusResultList);
    }

    public static ExtraitCorpusDef getExtraitCorpusDef(FicheFilter ficheFilter, IntituleClause intituleClause) {
        return new InternalExtraitCorpusDef(ficheFilter, intituleClause);
    }

    public static ExtraitThesaurusDef getExtraitThesaurusDef(MotcleFilter motcleFilter) {
        return new InternalExtraitThesaurusDef(motcleFilter);
    }

    public static ExtraitAlbumDef getExtraitAlbumDef(IllustrationFilter illustrationFilter) {
        return new InternalExtraitAlbumDef(illustrationFilter);
    }

    public static ExtraitAddendaDef getExtraitAddendaDef(DocumentFilter documentFilter) {
        return new InternalExtraitAddendaDef(documentFilter);
    }


    public static List<ExtraitCorpusResult> runExtraitCorpusDef(List<ExtraitCorpusDef> extraitCorpusDefList, ExtractionContext extractionContext) {
        if (extraitCorpusDefList.isEmpty()) {
            return BuilderUtils.EMPTY_EXTRAITCORPUSRESULTLIST;
        }
        Fiches selectedFiches = extractionContext.getSelectedFiches();
        List<FicheMeta> bufferList = null;
        if (selectedFiches != null) {
            bufferList = CorpusUtils.toList(selectedFiches);
        }
        Lang defaultLang = LocalisationUtils.getDefaultLang(extractionContext.getLangContext());
        List<ExtraitCorpusResult> list = new ArrayList<ExtraitCorpusResult>();
        for (ExtraitCorpusDef extraitCorpusDef : extraitCorpusDefList) {
            SelectionContext selectionContext = SelectionContextBuilder.init(extractionContext, defaultLang)
                    .setSubsetAccessPredicate(extractionContext.getSubsetAccessPredicate())
                    .toSelectionContext();
            FicheSelector ficheSelector = FicheSelectorBuilder.init(selectionContext)
                    .addAll(extraitCorpusDef.getEntryList())
                    .toFicheSelector();
            List<Corpus> corpusList = ficheSelector.getCorpusList();
            if (corpusList.isEmpty()) {
                continue;
            }
            CorpusExtractor corpusExtractor = extractionContext.getExtractorProvider().getCorpusExtractor(extraitCorpusDef);
            if (bufferList != null) {
                for (FicheMeta ficheMeta : bufferList) {
                    if (ficheSelector.test(ficheMeta)) {
                        corpusExtractor.add(ficheMeta, null);
                    }
                }
            } else {
                for (Corpus corpus : corpusList) {
                    for (FicheMeta ficheMeta : corpus.getFicheMetaList()) {
                        if (ficheSelector.test(ficheMeta)) {
                            corpusExtractor.add(ficheMeta, null);
                        }
                    }
                }
            }
            for (ExtraitCorpusResult extraitCorpusResult : corpusExtractor.getExtraitCorpusResultList()) {
                list.add(extraitCorpusResult);
            }
        }
        int size = list.size();
        if (size == 0) {
            return BuilderUtils.EMPTY_EXTRAITCORPUSRESULTLIST;
        }
        return list;
    }

    public static List<ExtraitThesaurusResult> runExtraitThesaurusDef(List<ExtraitThesaurusDef> extraitThesaurusDefList, ExtractionContext extractionContext) {
        if (extraitThesaurusDefList.isEmpty()) {
            return BuilderUtils.EMPTY_EXTRAITTHESAURUSRESULTLIST;
        }
        List<ExtraitThesaurusResult> list = new ArrayList<ExtraitThesaurusResult>();
        SelectionContext selectionContext = SelectionContextBuilder.init(extractionContext, LocalisationUtils.getDefaultLang(extractionContext.getLangContext()))
                .setSubsetAccessPredicate(extractionContext.getSubsetAccessPredicate())
                .toSelectionContext();
        for (ExtraitThesaurusDef extraitThesaurusDef : extraitThesaurusDefList) {
            MotcleSelector motcleSelector = MotcleSelectorBuilder.init(selectionContext).addAll(extraitThesaurusDef.getEntryList()).toMotcleSelector();
            List<Thesaurus> thesaurusList = motcleSelector.getThesaurusList();
            if (thesaurusList.isEmpty()) {
                continue;
            }
            ThesaurusExtractor thesaurusExtractor = extractionContext.getExtractorProvider().getThesaurusExtractor(extraitThesaurusDef);
            for (Thesaurus thesaurus : thesaurusList) {
                for (Motcle motcle : thesaurus.getMotcleList()) {
                    if (motcleSelector.test(motcle)) {
                        thesaurusExtractor.add(motcle, null);
                    }
                }
            }
            ExtraitThesaurusResult extraitThesaurusResult = thesaurusExtractor.getExtraitThesaurusResult();
            if (extraitThesaurusResult.getThesaurusCount() > 0) {
                list.add(extraitThesaurusResult);
            }
        }
        int size = list.size();
        if (size == 0) {
            return BuilderUtils.EMPTY_EXTRAITTHESAURUSRESULTLIST;
        }
        return list;
    }

    private static boolean testCharacters(String s) {
        int longueur = s.length();
        for (int i = 0; i < longueur; i++) {
            char carac = s.charAt(i);
            if (!isCharacter(carac)) {
                if (i == 0) {
                    return false;
                }
                if (!isNumber(carac)) {
                    if (i == (longueur - 1)) {
                        return false;
                    }
                    if (!isSpecialChar(carac)) {
                        return false;
                    }
                }

            }
        }
        return true;
    }

    private static boolean isCharacter(char carac) {
        return (((carac >= 'a') && (carac <= 'z')) || ((carac >= 'A') && (carac <= 'Z')));
    }

    private static boolean isNumber(char carac) {
        return ((carac >= '0') && (carac <= '9'));
    }

    private static boolean isSpecialChar(char carac) {
        switch (carac) {
            case '-':
            case '_':
                return true;
            default:
                return false;
        }
    }


    private static class InternalExtraitCorpusDef implements ExtraitCorpusDef {

        private final IntituleClause intituleClause;
        private final FicheFilter ficheFilter;

        private InternalExtraitCorpusDef(FicheFilter ficheFilter, IntituleClause intituleClause) {
            this.ficheFilter = ficheFilter;
            this.intituleClause = intituleClause;
        }

        @Override
        public String getTagName() {
            return ExtractionConstants.EXTRAITCORPUS_DEFAULT_TAGNAME;
        }

        @Override
        public boolean isMaster() {
            return false;
        }

        @Override
        public GroupClause getGroupClause() {
            return null;
        }

        @Override
        public PackClause getPackClause() {
            return null;
        }

        @Override
        public String getExtraitName() {
            return null;
        }

        @Override
        public IntituleClause getIntituleClause() {
            return intituleClause;
        }

        @Override
        public FicheFilter getFicheFilter() {
            return ficheFilter;
        }

        @Override
        public List<FicheCondition.Entry> getEntryList() {
            return SelectionUtils.EMPTY_FICHECONDITIONENTRYLIST;
        }

        @Override
        public boolean isDescendantAxis() {
            return false;
        }

    }


    private static class InternalExtraitThesaurusDef implements ExtraitThesaurusDef {

        private final MotcleFilter motcleFilter;

        private InternalExtraitThesaurusDef(MotcleFilter motcleFilter) {
            this.motcleFilter = motcleFilter;
        }

        @Override
        public String getTagName() {
            return ExtractionConstants.EXTRAITTHESAURUS_DEFAULT_TAGNAME;
        }

        @Override
        public String getExtraitName() {
            return null;
        }

        @Override
        public boolean isMaster() {
            return false;
        }

        @Override
        public MotcleFilter getMotcleFilter() {
            return motcleFilter;
        }

        @Override
        public List<MotcleCondition.Entry> getEntryList() {
            return SelectionUtils.EMPTY_MOTCLECONDITIONENTRYLIST;
        }

    }


    private static class InternalExtraitAlbumDef implements ExtraitAlbumDef {

        private final IllustrationFilter illustrationFilter;

        private InternalExtraitAlbumDef(IllustrationFilter illustrationFilter) {
            this.illustrationFilter = illustrationFilter;
        }

        @Override
        public String getTagName() {
            return ExtractionConstants.EXTRAITALBUM_DEFAULT_TAGNAME;
        }

        @Override
        public String getExtraitName() {
            return null;
        }

        @Override
        public IllustrationFilter getIllustrationFilter() {
            return illustrationFilter;
        }

        @Override
        public List<IllustrationCondition.Entry> getEntryList() {
            return SelectionUtils.EMPTY_ILLUSTRATIONCONDITIONENTRYLIST;
        }

    }


    private static class InternalExtraitAddendaDef implements ExtraitAddendaDef {

        private final DocumentFilter documentFilter;

        private InternalExtraitAddendaDef(DocumentFilter documentFilter) {
            this.documentFilter = documentFilter;
        }

        @Override
        public String getTagName() {
            return ExtractionConstants.EXTRAITALBUM_DEFAULT_TAGNAME;
        }

        @Override
        public String getExtraitName() {
            return null;
        }

        @Override
        public DocumentFilter getDocumentFilter() {
            return documentFilter;
        }

        @Override
        public List<DocumentCondition.Entry> getEntryList() {
            return SelectionUtils.EMPTY_DOCUMENTCONDITIONENTRYLIST;
        }

    }


    private static class SimpleExtractionSource implements ExtractionSource {

        private final Object dynamicObject;

        private SimpleExtractionSource(Object dynamicObject) {
            this.dynamicObject = dynamicObject;
        }

        @Override
        public Object getDynamicObject() {
            return dynamicObject;
        }

        @Override
        public List<ExtraitCorpusResult> getStaticExtraitCorpusResultList() {
            return EMPTY_EXTRAITCORPUSRESULTLIST;
        }

        @Override
        public List<ExtraitThesaurusResult> getStaticExtraitThesaurusResultList() {
            return EMPTY_EXTRAITTHESAURUSRESULTLIST;
        }

    }


    private static class CompleteExtractionSource implements ExtractionSource {

        private final Object dynamicObject;
        private final List<ExtraitCorpusResult> extraitCorpusResultList;
        private final List<ExtraitThesaurusResult> extraitThesaurusResultList;

        private CompleteExtractionSource(Object dynamicObject, List<ExtraitCorpusResult> extraitCorpusResultList, List<ExtraitThesaurusResult> extraitThesaurusResultList) {
            this.dynamicObject = dynamicObject;
            this.extraitCorpusResultList = extraitCorpusResultList;
            this.extraitThesaurusResultList = extraitThesaurusResultList;
        }

        @Override
        public Object getDynamicObject() {
            return dynamicObject;
        }

        @Override
        public List<ExtraitCorpusResult> getStaticExtraitCorpusResultList() {
            return extraitCorpusResultList;
        }

        @Override
        public List<ExtraitThesaurusResult> getStaticExtraitThesaurusResultList() {
            return extraitThesaurusResultList;
        }

    }

    public static List<ExtraitCorpusDef> wrap(ExtraitCorpusDef[] array) {
        return new ExtraitCorpusDefList(array);
    }


    private static class ExtraitCorpusDefList extends AbstractList<ExtraitCorpusDef> implements RandomAccess {

        private final ExtraitCorpusDef[] array;

        private ExtraitCorpusDefList(ExtraitCorpusDef[] array) {
            this.array = array;
        }

        @Override
        public int size() {
            return array.length;
        }

        @Override
        public ExtraitCorpusDef get(int index) {
            return array[index];
        }

    }

    public static List<FicheRattachementDef> wrap(FicheRattachementDef[] array) {
        return new FicheRattachementDefList(array);
    }


    private static class FicheRattachementDefList extends AbstractList<FicheRattachementDef> implements RandomAccess {

        private final FicheRattachementDef[] array;

        private FicheRattachementDefList(FicheRattachementDef[] array) {
            this.array = array;
        }

        @Override
        public int size() {
            return array.length;
        }

        @Override
        public FicheRattachementDef get(int index) {
            return array[index];
        }

    }

}
