/* FichothequeLib_Tools - Copyright (c) 2013-2017 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.fichotheque.tools.importation.dom;

import java.text.ParseException;
import net.fichotheque.Fichotheque;
import net.fichotheque.SubsetKey;
import net.fichotheque.corpus.Corpus;
import net.fichotheque.corpus.FicheMeta;
import net.fichotheque.corpus.fiche.ContentChecker;
import net.fichotheque.corpus.metadata.FieldKey;
import net.fichotheque.importation.CorpusImport;
import net.fichotheque.tools.corpus.FicheChangeBuilder;
import net.fichotheque.tools.dom.FicheDOMReader;
import net.fichotheque.tools.importation.CorpusImportBuilder;
import net.fichotheque.tools.importation.ImportationUtils;
import net.fichotheque.tools.importation.LiensImportBuilder;
import net.mapeadores.util.attr.AttributeChangeBuilder;
import net.mapeadores.util.exceptions.SwitchException;
import net.mapeadores.util.primitives.FuzzyDate;
import net.mapeadores.util.xml.DOMUtils;
import net.mapeadores.util.xml.ElementHandler;
import net.mapeadores.util.xml.XMLUtils;
import org.w3c.dom.Element;


/**
 *
 * @author Vincent Calame
 */
public class CorpusImportDOMReader {

    private final CorpusImportBuilder corpusImportBuilder;

    public CorpusImportDOMReader(CorpusImportBuilder corpusImportBuilder) {
        this.corpusImportBuilder = corpusImportBuilder;
    }

    public CorpusImportBuilder getCorpusImportBuilder() {
        return corpusImportBuilder;
    }

    public void readCorpusImport(Element element_xml) {
        ElementHandler elementHandler;
        String type = corpusImportBuilder.getType();
        if (type.equals(CorpusImport.CHANGE_TYPE)) {
            elementHandler = new ChangeHandler();
        } else if (type.equals(CorpusImport.CREATION_TYPE)) {
            elementHandler = new CreationHandler();
        } else if (type.equals(CorpusImport.REMOVE_TYPE)) {
            elementHandler = new RemoveHandler();
        } else {
            throw new SwitchException("Unknown type: " + type);
        }
        DOMUtils.readChildren(element_xml, elementHandler);

    }

    public static CorpusImportDOMReader fromRootElement(Fichotheque fichotheque, ContentChecker contentChecker, Element element) {
        String corpusName = element.getAttribute("corpus");
        try {
            SubsetKey corpusKey = SubsetKey.parse(SubsetKey.CATEGORY_CORPUS, corpusName);
            Corpus corpus = (Corpus) fichotheque.getSubset(corpusKey);
            if (corpus == null) {
                return null;
            }
            String type = element.getAttribute("type");
            if (!ImportationUtils.testCorpusImportType(type)) {
                return null;
            }
            CorpusImportBuilder corpusImportBuilder = CorpusImportBuilder.newInstance(contentChecker, corpus, type);
            return new CorpusImportDOMReader(corpusImportBuilder);
        } catch (ParseException pe) {
            return null;
        }
    }

    public static FicheImportHandler newFicheImportHandler(Fichotheque fichotheque, ContentChecker contentChecker) {
        FicheImportHandler ficheChangeHandler = new FicheImportHandler(fichotheque, contentChecker);
        ficheChangeHandler.clear();
        return ficheChangeHandler;
    }


    private class ChangeHandler implements ElementHandler {

        private final FicheImportHandler ficheImportHandler;

        private ChangeHandler() {
            this.ficheImportHandler = new FicheImportHandler(corpusImportBuilder.getCorpus().getFichotheque(), corpusImportBuilder.getContentChecker());
        }

        @Override
        public void readElement(Element element) {
            String tagName = element.getTagName();
            if (tagName.equals("ficheimport")) {
                String idString = element.getAttribute("id");
                try {
                    int id = Integer.parseInt(idString);
                    FicheMeta ficheMeta = corpusImportBuilder.getCorpus().getFicheMetaById(id);
                    if (ficheMeta != null) {
                        ficheImportHandler.clear();
                        DOMUtils.readChildren(element, ficheImportHandler);
                        corpusImportBuilder.addChange(ficheMeta, ficheImportHandler.getFicheChangeBuilder(), ficheImportHandler.getAttributeChangeBuilder(), ficheImportHandler.getLienImportBuilder(), ficheImportHandler.getCreationDate());
                    }
                } catch (NumberFormatException nfe) {
                }
            }
        }

    }


    private class CreationHandler implements ElementHandler {

        private final FicheImportHandler ficheImportHandler;

        private CreationHandler() {
            this.ficheImportHandler = new FicheImportHandler(corpusImportBuilder.getCorpus().getFichotheque(), corpusImportBuilder.getContentChecker());
        }

        @Override
        public void readElement(Element element) {
            String tagName = element.getTagName();
            if (tagName.equals("ficheimport")) {
                String idString = element.getAttribute("id");
                int newId = -1;
                if (idString.length() > 0) {
                    try {
                        newId = Integer.parseInt(idString);
                    } catch (NumberFormatException nfe) {
                    }
                }
                ficheImportHandler.clear();
                DOMUtils.readChildren(element, ficheImportHandler);
                corpusImportBuilder.addCreation(newId, ficheImportHandler.getFicheChangeBuilder(), ficheImportHandler.getAttributeChangeBuilder(), ficheImportHandler.getLienImportBuilder(), ficheImportHandler.getCreationDate());
            }
        }

    }


    private class RemoveHandler implements ElementHandler {

        private RemoveHandler() {
        }

        @Override
        public void readElement(Element element) {
            String tagName = element.getTagName();
            if (tagName.equals("ficheimport")) {
                String idString = element.getAttribute("id");
                try {
                    int id = Integer.parseInt(idString);
                    FicheMeta ficheMeta = corpusImportBuilder.getCorpus().getFicheMetaById(id);
                    if (ficheMeta != null) {
                        corpusImportBuilder.addRemove(ficheMeta);
                    }
                } catch (NumberFormatException nfe) {
                }
            }
        }

    }


    public static class FicheImportHandler implements ElementHandler {

        private final FicheDOMReader ficheDOMReader;
        private final LiensImportDOMReader liensImportDOMReader;
        private FicheChangeBuilder ficheChangeBuilder;
        private AttributeChangeBuilder attributeChangeBuilder;
        private LiensImportBuilder liensImportBuilder;
        private FuzzyDate creationDate;

        private FicheImportHandler(Fichotheque fichotheque, ContentChecker contentChecker) {
            this.liensImportDOMReader = new LiensImportDOMReader(fichotheque);
            ficheDOMReader = new FicheDOMReader(contentChecker);
            ficheDOMReader.setAppend(true);
        }

        @Override
        public void readElement(Element element) {
            String tagName = element.getTagName();
            if (tagName.equals("fiche")) {
                ficheChangeBuilder.appendFiche(ficheDOMReader.readFiche(element));
            } else if (tagName.equals("attr")) {
                ImportationDomUtils.readAttrElement(element, attributeChangeBuilder);
            } else if (tagName.equals("chrono")) {
                String creationString = element.getAttribute("creation");
                if (creationString.length() > 0) {
                    try {
                        creationDate = FuzzyDate.parse(creationString);
                    } catch (ParseException die) {

                    }
                }
            } else if (tagName.equals("removed")) {
                RemovedHandler removedHandler = new RemovedHandler(ficheChangeBuilder);
                DOMUtils.readChildren(element, removedHandler);
            } else {
                liensImportDOMReader.readLiensImport(liensImportBuilder, element);
            }
        }

        private void clear() {
            ficheChangeBuilder = new FicheChangeBuilder();
            attributeChangeBuilder = new AttributeChangeBuilder();
            liensImportBuilder = new LiensImportBuilder();
            creationDate = null;
        }

        public FicheChangeBuilder getFicheChangeBuilder() {
            return ficheChangeBuilder;
        }

        public LiensImportBuilder getLienImportBuilder() {
            return liensImportBuilder;
        }

        public AttributeChangeBuilder getAttributeChangeBuilder() {
            return attributeChangeBuilder;
        }

        public FuzzyDate getCreationDate() {
            return creationDate;
        }

    }


    public static class RemovedHandler implements ElementHandler {

        private final FicheChangeBuilder ficheChangeBuilder;

        private RemovedHandler(FicheChangeBuilder ficheChangeBuilder) {
            this.ficheChangeBuilder = ficheChangeBuilder;
        }

        @Override
        public void readElement(Element element) {
            String tagName = element.getTagName();
            if (tagName.equals("key")) {
                try {
                    FieldKey fieldKey = FieldKey.parse(XMLUtils.getData(element));
                    ficheChangeBuilder.removeField(fieldKey);
                } catch (ParseException pe) {

                }
            }
        }

    }

}
